from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.action_type import ActionType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ActionLiteDoc")


@_attrs_define
class ActionLiteDoc:
    """
    Attributes:
        field_id (str):
        type_ (ActionType):
        name (str):
        user_id (str):
        created_at (datetime.datetime):
        version (float | Unset):  Default: 0.0.
        user_manual (str | Unset):  Default: ''.
        description (str | Unset):  Default: ''.
        is_saved (bool | Unset):  Default: False.
        is_published (bool | Unset):  Default: False.
        shared_with (list[str] | None | Unset):
        updated_at (datetime.datetime | None | Unset):
        time_saved_per_run (float | None | Unset):  Default: 0.5.
        dollar_cost_per_hour (float | None | Unset):  Default: 50.0.
    """

    field_id: str
    type_: ActionType
    name: str
    user_id: str
    created_at: datetime.datetime
    version: float | Unset = 0.0
    user_manual: str | Unset = ""
    description: str | Unset = ""
    is_saved: bool | Unset = False
    is_published: bool | Unset = False
    shared_with: list[str] | None | Unset = UNSET
    updated_at: datetime.datetime | None | Unset = UNSET
    time_saved_per_run: float | None | Unset = 0.5
    dollar_cost_per_hour: float | None | Unset = 50.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_id = self.field_id

        type_ = self.type_.value

        name = self.name

        user_id = self.user_id

        created_at = self.created_at.isoformat()

        version = self.version

        user_manual = self.user_manual

        description = self.description

        is_saved = self.is_saved

        is_published = self.is_published

        shared_with: list[str] | None | Unset
        if isinstance(self.shared_with, Unset):
            shared_with = UNSET
        elif isinstance(self.shared_with, list):
            shared_with = self.shared_with

        else:
            shared_with = self.shared_with

        updated_at: None | str | Unset
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        time_saved_per_run: float | None | Unset
        if isinstance(self.time_saved_per_run, Unset):
            time_saved_per_run = UNSET
        else:
            time_saved_per_run = self.time_saved_per_run

        dollar_cost_per_hour: float | None | Unset
        if isinstance(self.dollar_cost_per_hour, Unset):
            dollar_cost_per_hour = UNSET
        else:
            dollar_cost_per_hour = self.dollar_cost_per_hour

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "_id": field_id,
                "type": type_,
                "name": name,
                "user_id": user_id,
                "created_at": created_at,
            }
        )
        if version is not UNSET:
            field_dict["version"] = version
        if user_manual is not UNSET:
            field_dict["user_manual"] = user_manual
        if description is not UNSET:
            field_dict["description"] = description
        if is_saved is not UNSET:
            field_dict["is_saved"] = is_saved
        if is_published is not UNSET:
            field_dict["is_published"] = is_published
        if shared_with is not UNSET:
            field_dict["shared_with"] = shared_with
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if time_saved_per_run is not UNSET:
            field_dict["time_saved_per_run"] = time_saved_per_run
        if dollar_cost_per_hour is not UNSET:
            field_dict["dollar_cost_per_hour"] = dollar_cost_per_hour

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        field_id = d.pop("_id")

        type_ = ActionType(d.pop("type"))

        name = d.pop("name")

        user_id = d.pop("user_id")

        created_at = isoparse(d.pop("created_at"))

        version = d.pop("version", UNSET)

        user_manual = d.pop("user_manual", UNSET)

        description = d.pop("description", UNSET)

        is_saved = d.pop("is_saved", UNSET)

        is_published = d.pop("is_published", UNSET)

        def _parse_shared_with(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                shared_with_type_0 = cast(list[str], data)

                return shared_with_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        shared_with = _parse_shared_with(d.pop("shared_with", UNSET))

        def _parse_updated_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)

                return updated_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        def _parse_time_saved_per_run(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        time_saved_per_run = _parse_time_saved_per_run(d.pop("time_saved_per_run", UNSET))

        def _parse_dollar_cost_per_hour(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        dollar_cost_per_hour = _parse_dollar_cost_per_hour(d.pop("dollar_cost_per_hour", UNSET))

        action_lite_doc = cls(
            field_id=field_id,
            type_=type_,
            name=name,
            user_id=user_id,
            created_at=created_at,
            version=version,
            user_manual=user_manual,
            description=description,
            is_saved=is_saved,
            is_published=is_published,
            shared_with=shared_with,
            updated_at=updated_at,
            time_saved_per_run=time_saved_per_run,
            dollar_cost_per_hour=dollar_cost_per_hour,
        )

        action_lite_doc.additional_properties = d
        return action_lite_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.file_list_response_schema import FileListResponseSchema
from ...models.http_validation_error import HTTPValidationError
from ...models.list_ingested_files_proxy_v1_ingestion_files_get_scope_type_0 import (
    ListIngestedFilesProxyV1IngestionFilesGetScopeType0,
)
from ...models.list_ingested_files_proxy_v1_ingestion_files_get_sort_by import (
    ListIngestedFilesProxyV1IngestionFilesGetSortBy,
)
from ...models.list_ingested_files_proxy_v1_ingestion_files_get_sort_order import (
    ListIngestedFilesProxyV1IngestionFilesGetSortOrder,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    filename_contains: None | str | Unset = UNSET,
    status: None | str | Unset = UNSET,
    weaviate_status: None | str | Unset = UNSET,
    graphrag_status: None | str | Unset = UNSET,
    start_date: None | str | Unset = UNSET,
    end_date: None | str | Unset = UNSET,
    sort_by: ListIngestedFilesProxyV1IngestionFilesGetSortBy
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT,
    sort_order: ListIngestedFilesProxyV1IngestionFilesGetSortOrder
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC,
    limit: int | Unset = 50,
    offset: int | Unset = 0,
    kb_id: None | str | Unset = UNSET,
    kb_name: None | str | Unset = UNSET,
    scope: ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_filename_contains: None | str | Unset
    if isinstance(filename_contains, Unset):
        json_filename_contains = UNSET
    else:
        json_filename_contains = filename_contains
    params["filename_contains"] = json_filename_contains

    json_status: None | str | Unset
    if isinstance(status, Unset):
        json_status = UNSET
    else:
        json_status = status
    params["status"] = json_status

    json_weaviate_status: None | str | Unset
    if isinstance(weaviate_status, Unset):
        json_weaviate_status = UNSET
    else:
        json_weaviate_status = weaviate_status
    params["weaviate_status"] = json_weaviate_status

    json_graphrag_status: None | str | Unset
    if isinstance(graphrag_status, Unset):
        json_graphrag_status = UNSET
    else:
        json_graphrag_status = graphrag_status
    params["graphrag_status"] = json_graphrag_status

    json_start_date: None | str | Unset
    if isinstance(start_date, Unset):
        json_start_date = UNSET
    else:
        json_start_date = start_date
    params["start_date"] = json_start_date

    json_end_date: None | str | Unset
    if isinstance(end_date, Unset):
        json_end_date = UNSET
    else:
        json_end_date = end_date
    params["end_date"] = json_end_date

    json_sort_by: str | Unset = UNSET
    if not isinstance(sort_by, Unset):
        json_sort_by = sort_by.value

    params["sort_by"] = json_sort_by

    json_sort_order: str | Unset = UNSET
    if not isinstance(sort_order, Unset):
        json_sort_order = sort_order.value

    params["sort_order"] = json_sort_order

    params["limit"] = limit

    params["offset"] = offset

    json_kb_id: None | str | Unset
    if isinstance(kb_id, Unset):
        json_kb_id = UNSET
    else:
        json_kb_id = kb_id
    params["kb_id"] = json_kb_id

    json_kb_name: None | str | Unset
    if isinstance(kb_name, Unset):
        json_kb_name = UNSET
    else:
        json_kb_name = kb_name
    params["kb_name"] = json_kb_name

    json_scope: None | str | Unset
    if isinstance(scope, Unset):
        json_scope = UNSET
    elif isinstance(scope, ListIngestedFilesProxyV1IngestionFilesGetScopeType0):
        json_scope = scope.value
    else:
        json_scope = scope
    params["scope"] = json_scope

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/ingestion/files",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> FileListResponseSchema | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = FileListResponseSchema.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[FileListResponseSchema | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    filename_contains: None | str | Unset = UNSET,
    status: None | str | Unset = UNSET,
    weaviate_status: None | str | Unset = UNSET,
    graphrag_status: None | str | Unset = UNSET,
    start_date: None | str | Unset = UNSET,
    end_date: None | str | Unset = UNSET,
    sort_by: ListIngestedFilesProxyV1IngestionFilesGetSortBy
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT,
    sort_order: ListIngestedFilesProxyV1IngestionFilesGetSortOrder
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC,
    limit: int | Unset = 50,
    offset: int | Unset = 0,
    kb_id: None | str | Unset = UNSET,
    kb_name: None | str | Unset = UNSET,
    scope: ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset = UNSET,
) -> Response[FileListResponseSchema | HTTPValidationError]:
    """List ingested files

     [Proxy] Forwards the request to the Ingestion Worker Service.

    Args:
        filename_contains (None | str | Unset): Search filename.
        status (None | str | Unset): Filter by status.
        weaviate_status (None | str | Unset):
        graphrag_status (None | str | Unset):
        start_date (None | str | Unset):
        end_date (None | str | Unset):
        sort_by (ListIngestedFilesProxyV1IngestionFilesGetSortBy | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT.
        sort_order (ListIngestedFilesProxyV1IngestionFilesGetSortOrder | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC.
        limit (int | Unset):  Default: 50.
        offset (int | Unset):  Default: 0.
        kb_id (None | str | Unset): Optional KB filter. If provided, lists files for that KB.
        kb_name (None | str | Unset): Optional KB name filter. Server resolves to kb_id if unique.
        scope (ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset): Filter scope:
            'personal', 'kb', 'all'. Default: 'all' (My Uploads).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[FileListResponseSchema | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        filename_contains=filename_contains,
        status=status,
        weaviate_status=weaviate_status,
        graphrag_status=graphrag_status,
        start_date=start_date,
        end_date=end_date,
        sort_by=sort_by,
        sort_order=sort_order,
        limit=limit,
        offset=offset,
        kb_id=kb_id,
        kb_name=kb_name,
        scope=scope,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    filename_contains: None | str | Unset = UNSET,
    status: None | str | Unset = UNSET,
    weaviate_status: None | str | Unset = UNSET,
    graphrag_status: None | str | Unset = UNSET,
    start_date: None | str | Unset = UNSET,
    end_date: None | str | Unset = UNSET,
    sort_by: ListIngestedFilesProxyV1IngestionFilesGetSortBy
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT,
    sort_order: ListIngestedFilesProxyV1IngestionFilesGetSortOrder
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC,
    limit: int | Unset = 50,
    offset: int | Unset = 0,
    kb_id: None | str | Unset = UNSET,
    kb_name: None | str | Unset = UNSET,
    scope: ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset = UNSET,
) -> FileListResponseSchema | HTTPValidationError | None:
    """List ingested files

     [Proxy] Forwards the request to the Ingestion Worker Service.

    Args:
        filename_contains (None | str | Unset): Search filename.
        status (None | str | Unset): Filter by status.
        weaviate_status (None | str | Unset):
        graphrag_status (None | str | Unset):
        start_date (None | str | Unset):
        end_date (None | str | Unset):
        sort_by (ListIngestedFilesProxyV1IngestionFilesGetSortBy | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT.
        sort_order (ListIngestedFilesProxyV1IngestionFilesGetSortOrder | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC.
        limit (int | Unset):  Default: 50.
        offset (int | Unset):  Default: 0.
        kb_id (None | str | Unset): Optional KB filter. If provided, lists files for that KB.
        kb_name (None | str | Unset): Optional KB name filter. Server resolves to kb_id if unique.
        scope (ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset): Filter scope:
            'personal', 'kb', 'all'. Default: 'all' (My Uploads).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        FileListResponseSchema | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        filename_contains=filename_contains,
        status=status,
        weaviate_status=weaviate_status,
        graphrag_status=graphrag_status,
        start_date=start_date,
        end_date=end_date,
        sort_by=sort_by,
        sort_order=sort_order,
        limit=limit,
        offset=offset,
        kb_id=kb_id,
        kb_name=kb_name,
        scope=scope,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    filename_contains: None | str | Unset = UNSET,
    status: None | str | Unset = UNSET,
    weaviate_status: None | str | Unset = UNSET,
    graphrag_status: None | str | Unset = UNSET,
    start_date: None | str | Unset = UNSET,
    end_date: None | str | Unset = UNSET,
    sort_by: ListIngestedFilesProxyV1IngestionFilesGetSortBy
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT,
    sort_order: ListIngestedFilesProxyV1IngestionFilesGetSortOrder
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC,
    limit: int | Unset = 50,
    offset: int | Unset = 0,
    kb_id: None | str | Unset = UNSET,
    kb_name: None | str | Unset = UNSET,
    scope: ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset = UNSET,
) -> Response[FileListResponseSchema | HTTPValidationError]:
    """List ingested files

     [Proxy] Forwards the request to the Ingestion Worker Service.

    Args:
        filename_contains (None | str | Unset): Search filename.
        status (None | str | Unset): Filter by status.
        weaviate_status (None | str | Unset):
        graphrag_status (None | str | Unset):
        start_date (None | str | Unset):
        end_date (None | str | Unset):
        sort_by (ListIngestedFilesProxyV1IngestionFilesGetSortBy | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT.
        sort_order (ListIngestedFilesProxyV1IngestionFilesGetSortOrder | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC.
        limit (int | Unset):  Default: 50.
        offset (int | Unset):  Default: 0.
        kb_id (None | str | Unset): Optional KB filter. If provided, lists files for that KB.
        kb_name (None | str | Unset): Optional KB name filter. Server resolves to kb_id if unique.
        scope (ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset): Filter scope:
            'personal', 'kb', 'all'. Default: 'all' (My Uploads).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[FileListResponseSchema | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        filename_contains=filename_contains,
        status=status,
        weaviate_status=weaviate_status,
        graphrag_status=graphrag_status,
        start_date=start_date,
        end_date=end_date,
        sort_by=sort_by,
        sort_order=sort_order,
        limit=limit,
        offset=offset,
        kb_id=kb_id,
        kb_name=kb_name,
        scope=scope,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    filename_contains: None | str | Unset = UNSET,
    status: None | str | Unset = UNSET,
    weaviate_status: None | str | Unset = UNSET,
    graphrag_status: None | str | Unset = UNSET,
    start_date: None | str | Unset = UNSET,
    end_date: None | str | Unset = UNSET,
    sort_by: ListIngestedFilesProxyV1IngestionFilesGetSortBy
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT,
    sort_order: ListIngestedFilesProxyV1IngestionFilesGetSortOrder
    | Unset = ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC,
    limit: int | Unset = 50,
    offset: int | Unset = 0,
    kb_id: None | str | Unset = UNSET,
    kb_name: None | str | Unset = UNSET,
    scope: ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset = UNSET,
) -> FileListResponseSchema | HTTPValidationError | None:
    """List ingested files

     [Proxy] Forwards the request to the Ingestion Worker Service.

    Args:
        filename_contains (None | str | Unset): Search filename.
        status (None | str | Unset): Filter by status.
        weaviate_status (None | str | Unset):
        graphrag_status (None | str | Unset):
        start_date (None | str | Unset):
        end_date (None | str | Unset):
        sort_by (ListIngestedFilesProxyV1IngestionFilesGetSortBy | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortBy.CREATED_AT.
        sort_order (ListIngestedFilesProxyV1IngestionFilesGetSortOrder | Unset):  Default:
            ListIngestedFilesProxyV1IngestionFilesGetSortOrder.DESC.
        limit (int | Unset):  Default: 50.
        offset (int | Unset):  Default: 0.
        kb_id (None | str | Unset): Optional KB filter. If provided, lists files for that KB.
        kb_name (None | str | Unset): Optional KB name filter. Server resolves to kb_id if unique.
        scope (ListIngestedFilesProxyV1IngestionFilesGetScopeType0 | None | Unset): Filter scope:
            'personal', 'kb', 'all'. Default: 'all' (My Uploads).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        FileListResponseSchema | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            filename_contains=filename_contains,
            status=status,
            weaviate_status=weaviate_status,
            graphrag_status=graphrag_status,
            start_date=start_date,
            end_date=end_date,
            sort_by=sort_by,
            sort_order=sort_order,
            limit=limit,
            offset=offset,
            kb_id=kb_id,
            kb_name=kb_name,
            scope=scope,
        )
    ).parsed



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.factor_factor_type_type_0 import FactorFactorTypeType0
from ..models.factor_status import FactorStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="Factor")


@_attrs_define
class Factor:
    """A MFA factor.

    Attributes:
        id (str):
        factor_type (FactorFactorTypeType0 | str):
        status (FactorStatus):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        friendly_name (None | str | Unset):
    """

    id: str
    factor_type: FactorFactorTypeType0 | str
    status: FactorStatus
    created_at: datetime.datetime
    updated_at: datetime.datetime
    friendly_name: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        factor_type: str
        if isinstance(self.factor_type, FactorFactorTypeType0):
            factor_type = self.factor_type.value
        else:
            factor_type = self.factor_type

        status = self.status.value

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        friendly_name: None | str | Unset
        if isinstance(self.friendly_name, Unset):
            friendly_name = UNSET
        else:
            friendly_name = self.friendly_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "factor_type": factor_type,
                "status": status,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if friendly_name is not UNSET:
            field_dict["friendly_name"] = friendly_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        def _parse_factor_type(data: object) -> FactorFactorTypeType0 | str:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                factor_type_type_0 = FactorFactorTypeType0(data)

                return factor_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(FactorFactorTypeType0 | str, data)

        factor_type = _parse_factor_type(d.pop("factor_type"))

        status = FactorStatus(d.pop("status"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        def _parse_friendly_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        friendly_name = _parse_friendly_name(d.pop("friendly_name", UNSET))

        factor = cls(
            id=id,
            factor_type=factor_type,
            status=status,
            created_at=created_at,
            updated_at=updated_at,
            friendly_name=friendly_name,
        )

        factor.additional_properties = d
        return factor

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.message import Message


T = TypeVar("T", bound="ChatUpdate")


@_attrs_define
class ChatUpdate:
    """
    Attributes:
        messages (list[Message] | None):
        title (None | str | Unset):
    """

    messages: list[Message] | None
    title: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        messages: list[dict[str, Any]] | None
        if isinstance(self.messages, list):
            messages = []
            for messages_type_0_item_data in self.messages:
                messages_type_0_item = messages_type_0_item_data.to_dict()
                messages.append(messages_type_0_item)

        else:
            messages = self.messages

        title: None | str | Unset
        if isinstance(self.title, Unset):
            title = UNSET
        else:
            title = self.title

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "messages": messages,
            }
        )
        if title is not UNSET:
            field_dict["title"] = title

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.message import Message

        d = dict(src_dict)

        def _parse_messages(data: object) -> list[Message] | None:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                messages_type_0 = []
                _messages_type_0 = data
                for messages_type_0_item_data in _messages_type_0:
                    messages_type_0_item = Message.from_dict(messages_type_0_item_data)

                    messages_type_0.append(messages_type_0_item)

                return messages_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[Message] | None, data)

        messages = _parse_messages(d.pop("messages"))

        def _parse_title(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        title = _parse_title(d.pop("title", UNSET))

        chat_update = cls(
            messages=messages,
            title=title,
        )

        chat_update.additional_properties = d
        return chat_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.body_ingest_file_async_v1_ingestion_file_async_post import BodyIngestFileAsyncV1IngestionFileAsyncPost
from ...models.http_validation_error import HTTPValidationError
from ...models.ingest_async_response import IngestAsyncResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: BodyIngestFileAsyncV1IngestionFileAsyncPost,
    kb_id: None | str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    json_kb_id: None | str | Unset
    if isinstance(kb_id, Unset):
        json_kb_id = UNSET
    else:
        json_kb_id = kb_id
    params["kb_id"] = json_kb_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/ingestion/file/async",
        "params": params,
    }

    _kwargs["files"] = body.to_multipart()

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | IngestAsyncResponse | None:
    if response.status_code == 200:
        response_200 = IngestAsyncResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | IngestAsyncResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    body: BodyIngestFileAsyncV1IngestionFileAsyncPost,
    kb_id: None | str | Unset = UNSET,
) -> Response[HTTPValidationError | IngestAsyncResponse]:
    """Ingest File Async

     [V2] Accepts a file for high-priority ingestion into the user's permanent knowledge base.
    1. Persists the file to the user's permanent S3 folder (e.g., files/<user_id>/).
    2. Stages a copy for the ingestion worker.
    3. Queues a job in Redis for the 'synced_files' collection.
    4. Logs the file in 'processed_files_log' to prevent duplicate ingestion by the nightly cron job.

    Args:
        kb_id (None | str | Unset): Optional knowledge base ID. If omitted, uses the caller's
            personal knowledge base.
        body (BodyIngestFileAsyncV1IngestionFileAsyncPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | IngestAsyncResponse]
    """

    kwargs = _get_kwargs(
        body=body,
        kb_id=kb_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    body: BodyIngestFileAsyncV1IngestionFileAsyncPost,
    kb_id: None | str | Unset = UNSET,
) -> HTTPValidationError | IngestAsyncResponse | None:
    """Ingest File Async

     [V2] Accepts a file for high-priority ingestion into the user's permanent knowledge base.
    1. Persists the file to the user's permanent S3 folder (e.g., files/<user_id>/).
    2. Stages a copy for the ingestion worker.
    3. Queues a job in Redis for the 'synced_files' collection.
    4. Logs the file in 'processed_files_log' to prevent duplicate ingestion by the nightly cron job.

    Args:
        kb_id (None | str | Unset): Optional knowledge base ID. If omitted, uses the caller's
            personal knowledge base.
        body (BodyIngestFileAsyncV1IngestionFileAsyncPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | IngestAsyncResponse
    """

    return sync_detailed(
        client=client,
        body=body,
        kb_id=kb_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    body: BodyIngestFileAsyncV1IngestionFileAsyncPost,
    kb_id: None | str | Unset = UNSET,
) -> Response[HTTPValidationError | IngestAsyncResponse]:
    """Ingest File Async

     [V2] Accepts a file for high-priority ingestion into the user's permanent knowledge base.
    1. Persists the file to the user's permanent S3 folder (e.g., files/<user_id>/).
    2. Stages a copy for the ingestion worker.
    3. Queues a job in Redis for the 'synced_files' collection.
    4. Logs the file in 'processed_files_log' to prevent duplicate ingestion by the nightly cron job.

    Args:
        kb_id (None | str | Unset): Optional knowledge base ID. If omitted, uses the caller's
            personal knowledge base.
        body (BodyIngestFileAsyncV1IngestionFileAsyncPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | IngestAsyncResponse]
    """

    kwargs = _get_kwargs(
        body=body,
        kb_id=kb_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    body: BodyIngestFileAsyncV1IngestionFileAsyncPost,
    kb_id: None | str | Unset = UNSET,
) -> HTTPValidationError | IngestAsyncResponse | None:
    """Ingest File Async

     [V2] Accepts a file for high-priority ingestion into the user's permanent knowledge base.
    1. Persists the file to the user's permanent S3 folder (e.g., files/<user_id>/).
    2. Stages a copy for the ingestion worker.
    3. Queues a job in Redis for the 'synced_files' collection.
    4. Logs the file in 'processed_files_log' to prevent duplicate ingestion by the nightly cron job.

    Args:
        kb_id (None | str | Unset): Optional knowledge base ID. If omitted, uses the caller's
            personal knowledge base.
        body (BodyIngestFileAsyncV1IngestionFileAsyncPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | IngestAsyncResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
            kb_id=kb_id,
        )
    ).parsed



from enum import Enum


class AppTypes(str, Enum):
    ADP = "adp"
    AIRTABLE = "airtable"
    ANON = "anon"
    ASANA = "asana"
    ASSEMBLYAI = "assemblyai"
    AWS = "aws"
    BOX = "box"
    CARTESIA = "cartesia"
    CHARGEBEE = "chargebee"
    CLAUDE = "claude"
    DATABRICKS = "databricks"
    DATADOG = "datadog"
    DATEV = "datev"
    DEEPGRAM = "deepgram"
    DEEPSEEK = "deepseek"
    DROPBOX = "dropbox"
    EXA = "exa"
    FIRECRAWL = "firecrawl"
    FLYIO = "flyio"
    GAMMA_AI = "gamma_ai"
    GEMINI = "gemini"
    GITHUB = "github"
    GOOGLE = "google"
    GOOGLE_ADS = "google_ads"
    GOOGLE_SERP = "google_serp"
    GRAFANA = "grafana"
    GROK = "grok"
    HUBSPOT = "hubspot"
    ICIMS = "icims"
    INTUIT = "intuit"
    JIRA = "jira"
    LOOKER = "looker"
    META = "meta"
    MUSO = "muso"
    NETSUITE = "netsuite"
    NORTHFLANK = "northflank"
    NOTION = "notion"
    OPENAI = "openai"
    OPUS = "opus"
    ORDWAY = "ordway"
    OUTLOOK = "outlook"
    PLAID = "plaid"
    REMOTION = "remotion"
    REPLICATE_FLUX = "replicate_flux"
    SALESFORCE = "salesforce"
    SAP = "sap"
    SIGMA_COMPUTING = "sigma_computing"
    SLACK = "slack"
    SLACK_USER = "slack_user"
    SNOWFLAKE = "snowflake"
    SPOTIFY = "spotify"
    STRIPE = "stripe"
    SUPERSET = "superset"
    THESYS = "thesys"
    TIPALTI = "tipalti"
    UKG = "ukg"
    UNSTRUCTUREDIO = "unstructuredio"
    VERCEL = "vercel"
    WHATSAPP = "whatsapp"
    WORKDAY = "workday"
    ZENDESK = "zendesk"
    ZOHO = "zoho"
    ZOOM = "zoom"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.partition_parameters_params_hi_res_model_name_type_0 import PartitionParametersParamsHiResModelNameType0
from ..models.partition_parameters_params_strategy_type_0 import PartitionParametersParamsStrategyType0
from ..types import UNSET, Unset

T = TypeVar("T", bound="PartitionParametersParams")


@_attrs_define
class PartitionParametersParams:
    """
    Attributes:
        chunking_strategy (Literal['by_title'] | None | Unset):
        combine_under_n_chars (int | None | Unset):
        coordinates (bool | None | Unset):
        encoding (None | str | Unset):
        extract_image_block_types (list[str] | None | Unset):
        gz_uncompressed_content_type (None | str | Unset):
        hi_res_model_name (None | PartitionParametersParamsHiResModelNameType0 | Unset):
        include_page_breaks (bool | None | Unset):
        languages (list[str] | None | Unset):
        max_characters (int | None | Unset):
        multipage_sections (bool | None | Unset):
        new_after_n_chars (int | None | Unset):
        output_format (None | str | Unset):
        overlap (int | None | Unset):
        overlap_all (bool | None | Unset):
        pdf_infer_table_structure (bool | None | Unset):
        skip_infer_table_types (list[str] | None | Unset):
        split_pdf_page (bool | None | Unset):
        strategy (None | PartitionParametersParamsStrategyType0 | Unset):
        xml_keep_tags (bool | None | Unset):
    """

    chunking_strategy: Literal["by_title"] | None | Unset = UNSET
    combine_under_n_chars: int | None | Unset = UNSET
    coordinates: bool | None | Unset = UNSET
    encoding: None | str | Unset = UNSET
    extract_image_block_types: list[str] | None | Unset = UNSET
    gz_uncompressed_content_type: None | str | Unset = UNSET
    hi_res_model_name: None | PartitionParametersParamsHiResModelNameType0 | Unset = UNSET
    include_page_breaks: bool | None | Unset = UNSET
    languages: list[str] | None | Unset = UNSET
    max_characters: int | None | Unset = UNSET
    multipage_sections: bool | None | Unset = UNSET
    new_after_n_chars: int | None | Unset = UNSET
    output_format: None | str | Unset = UNSET
    overlap: int | None | Unset = UNSET
    overlap_all: bool | None | Unset = UNSET
    pdf_infer_table_structure: bool | None | Unset = UNSET
    skip_infer_table_types: list[str] | None | Unset = UNSET
    split_pdf_page: bool | None | Unset = UNSET
    strategy: None | PartitionParametersParamsStrategyType0 | Unset = UNSET
    xml_keep_tags: bool | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        chunking_strategy: Literal["by_title"] | None | Unset
        if isinstance(self.chunking_strategy, Unset):
            chunking_strategy = UNSET
        else:
            chunking_strategy = self.chunking_strategy

        combine_under_n_chars: int | None | Unset
        if isinstance(self.combine_under_n_chars, Unset):
            combine_under_n_chars = UNSET
        else:
            combine_under_n_chars = self.combine_under_n_chars

        coordinates: bool | None | Unset
        if isinstance(self.coordinates, Unset):
            coordinates = UNSET
        else:
            coordinates = self.coordinates

        encoding: None | str | Unset
        if isinstance(self.encoding, Unset):
            encoding = UNSET
        else:
            encoding = self.encoding

        extract_image_block_types: list[str] | None | Unset
        if isinstance(self.extract_image_block_types, Unset):
            extract_image_block_types = UNSET
        elif isinstance(self.extract_image_block_types, list):
            extract_image_block_types = self.extract_image_block_types

        else:
            extract_image_block_types = self.extract_image_block_types

        gz_uncompressed_content_type: None | str | Unset
        if isinstance(self.gz_uncompressed_content_type, Unset):
            gz_uncompressed_content_type = UNSET
        else:
            gz_uncompressed_content_type = self.gz_uncompressed_content_type

        hi_res_model_name: None | str | Unset
        if isinstance(self.hi_res_model_name, Unset):
            hi_res_model_name = UNSET
        elif isinstance(self.hi_res_model_name, PartitionParametersParamsHiResModelNameType0):
            hi_res_model_name = self.hi_res_model_name.value
        else:
            hi_res_model_name = self.hi_res_model_name

        include_page_breaks: bool | None | Unset
        if isinstance(self.include_page_breaks, Unset):
            include_page_breaks = UNSET
        else:
            include_page_breaks = self.include_page_breaks

        languages: list[str] | None | Unset
        if isinstance(self.languages, Unset):
            languages = UNSET
        elif isinstance(self.languages, list):
            languages = self.languages

        else:
            languages = self.languages

        max_characters: int | None | Unset
        if isinstance(self.max_characters, Unset):
            max_characters = UNSET
        else:
            max_characters = self.max_characters

        multipage_sections: bool | None | Unset
        if isinstance(self.multipage_sections, Unset):
            multipage_sections = UNSET
        else:
            multipage_sections = self.multipage_sections

        new_after_n_chars: int | None | Unset
        if isinstance(self.new_after_n_chars, Unset):
            new_after_n_chars = UNSET
        else:
            new_after_n_chars = self.new_after_n_chars

        output_format: None | str | Unset
        if isinstance(self.output_format, Unset):
            output_format = UNSET
        else:
            output_format = self.output_format

        overlap: int | None | Unset
        if isinstance(self.overlap, Unset):
            overlap = UNSET
        else:
            overlap = self.overlap

        overlap_all: bool | None | Unset
        if isinstance(self.overlap_all, Unset):
            overlap_all = UNSET
        else:
            overlap_all = self.overlap_all

        pdf_infer_table_structure: bool | None | Unset
        if isinstance(self.pdf_infer_table_structure, Unset):
            pdf_infer_table_structure = UNSET
        else:
            pdf_infer_table_structure = self.pdf_infer_table_structure

        skip_infer_table_types: list[str] | None | Unset
        if isinstance(self.skip_infer_table_types, Unset):
            skip_infer_table_types = UNSET
        elif isinstance(self.skip_infer_table_types, list):
            skip_infer_table_types = self.skip_infer_table_types

        else:
            skip_infer_table_types = self.skip_infer_table_types

        split_pdf_page: bool | None | Unset
        if isinstance(self.split_pdf_page, Unset):
            split_pdf_page = UNSET
        else:
            split_pdf_page = self.split_pdf_page

        strategy: None | str | Unset
        if isinstance(self.strategy, Unset):
            strategy = UNSET
        elif isinstance(self.strategy, PartitionParametersParamsStrategyType0):
            strategy = self.strategy.value
        else:
            strategy = self.strategy

        xml_keep_tags: bool | None | Unset
        if isinstance(self.xml_keep_tags, Unset):
            xml_keep_tags = UNSET
        else:
            xml_keep_tags = self.xml_keep_tags

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if chunking_strategy is not UNSET:
            field_dict["chunking_strategy"] = chunking_strategy
        if combine_under_n_chars is not UNSET:
            field_dict["combine_under_n_chars"] = combine_under_n_chars
        if coordinates is not UNSET:
            field_dict["coordinates"] = coordinates
        if encoding is not UNSET:
            field_dict["encoding"] = encoding
        if extract_image_block_types is not UNSET:
            field_dict["extract_image_block_types"] = extract_image_block_types
        if gz_uncompressed_content_type is not UNSET:
            field_dict["gz_uncompressed_content_type"] = gz_uncompressed_content_type
        if hi_res_model_name is not UNSET:
            field_dict["hi_res_model_name"] = hi_res_model_name
        if include_page_breaks is not UNSET:
            field_dict["include_page_breaks"] = include_page_breaks
        if languages is not UNSET:
            field_dict["languages"] = languages
        if max_characters is not UNSET:
            field_dict["max_characters"] = max_characters
        if multipage_sections is not UNSET:
            field_dict["multipage_sections"] = multipage_sections
        if new_after_n_chars is not UNSET:
            field_dict["new_after_n_chars"] = new_after_n_chars
        if output_format is not UNSET:
            field_dict["output_format"] = output_format
        if overlap is not UNSET:
            field_dict["overlap"] = overlap
        if overlap_all is not UNSET:
            field_dict["overlap_all"] = overlap_all
        if pdf_infer_table_structure is not UNSET:
            field_dict["pdf_infer_table_structure"] = pdf_infer_table_structure
        if skip_infer_table_types is not UNSET:
            field_dict["skip_infer_table_types"] = skip_infer_table_types
        if split_pdf_page is not UNSET:
            field_dict["split_pdf_page"] = split_pdf_page
        if strategy is not UNSET:
            field_dict["strategy"] = strategy
        if xml_keep_tags is not UNSET:
            field_dict["xml_keep_tags"] = xml_keep_tags

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_chunking_strategy(data: object) -> Literal["by_title"] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            chunking_strategy_type_0 = cast(Literal["by_title"], data)
            if chunking_strategy_type_0 != "by_title":
                raise ValueError(
                    f"chunking_strategy_type_0 must match const 'by_title', got '{chunking_strategy_type_0}'"
                )
            return chunking_strategy_type_0
            return cast(Literal["by_title"] | None | Unset, data)

        chunking_strategy = _parse_chunking_strategy(d.pop("chunking_strategy", UNSET))

        def _parse_combine_under_n_chars(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        combine_under_n_chars = _parse_combine_under_n_chars(d.pop("combine_under_n_chars", UNSET))

        def _parse_coordinates(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        coordinates = _parse_coordinates(d.pop("coordinates", UNSET))

        def _parse_encoding(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        encoding = _parse_encoding(d.pop("encoding", UNSET))

        def _parse_extract_image_block_types(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                extract_image_block_types_type_0 = cast(list[str], data)

                return extract_image_block_types_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        extract_image_block_types = _parse_extract_image_block_types(d.pop("extract_image_block_types", UNSET))

        def _parse_gz_uncompressed_content_type(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        gz_uncompressed_content_type = _parse_gz_uncompressed_content_type(d.pop("gz_uncompressed_content_type", UNSET))

        def _parse_hi_res_model_name(data: object) -> None | PartitionParametersParamsHiResModelNameType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                hi_res_model_name_type_0 = PartitionParametersParamsHiResModelNameType0(data)

                return hi_res_model_name_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | PartitionParametersParamsHiResModelNameType0 | Unset, data)

        hi_res_model_name = _parse_hi_res_model_name(d.pop("hi_res_model_name", UNSET))

        def _parse_include_page_breaks(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        include_page_breaks = _parse_include_page_breaks(d.pop("include_page_breaks", UNSET))

        def _parse_languages(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                languages_type_0 = cast(list[str], data)

                return languages_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        languages = _parse_languages(d.pop("languages", UNSET))

        def _parse_max_characters(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        max_characters = _parse_max_characters(d.pop("max_characters", UNSET))

        def _parse_multipage_sections(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        multipage_sections = _parse_multipage_sections(d.pop("multipage_sections", UNSET))

        def _parse_new_after_n_chars(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        new_after_n_chars = _parse_new_after_n_chars(d.pop("new_after_n_chars", UNSET))

        def _parse_output_format(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        output_format = _parse_output_format(d.pop("output_format", UNSET))

        def _parse_overlap(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        overlap = _parse_overlap(d.pop("overlap", UNSET))

        def _parse_overlap_all(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        overlap_all = _parse_overlap_all(d.pop("overlap_all", UNSET))

        def _parse_pdf_infer_table_structure(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        pdf_infer_table_structure = _parse_pdf_infer_table_structure(d.pop("pdf_infer_table_structure", UNSET))

        def _parse_skip_infer_table_types(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                skip_infer_table_types_type_0 = cast(list[str], data)

                return skip_infer_table_types_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        skip_infer_table_types = _parse_skip_infer_table_types(d.pop("skip_infer_table_types", UNSET))

        def _parse_split_pdf_page(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        split_pdf_page = _parse_split_pdf_page(d.pop("split_pdf_page", UNSET))

        def _parse_strategy(data: object) -> None | PartitionParametersParamsStrategyType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                strategy_type_0 = PartitionParametersParamsStrategyType0(data)

                return strategy_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | PartitionParametersParamsStrategyType0 | Unset, data)

        strategy = _parse_strategy(d.pop("strategy", UNSET))

        def _parse_xml_keep_tags(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        xml_keep_tags = _parse_xml_keep_tags(d.pop("xml_keep_tags", UNSET))

        partition_parameters_params = cls(
            chunking_strategy=chunking_strategy,
            combine_under_n_chars=combine_under_n_chars,
            coordinates=coordinates,
            encoding=encoding,
            extract_image_block_types=extract_image_block_types,
            gz_uncompressed_content_type=gz_uncompressed_content_type,
            hi_res_model_name=hi_res_model_name,
            include_page_breaks=include_page_breaks,
            languages=languages,
            max_characters=max_characters,
            multipage_sections=multipage_sections,
            new_after_n_chars=new_after_n_chars,
            output_format=output_format,
            overlap=overlap,
            overlap_all=overlap_all,
            pdf_infer_table_structure=pdf_infer_table_structure,
            skip_infer_table_types=skip_infer_table_types,
            split_pdf_page=split_pdf_page,
            strategy=strategy,
            xml_keep_tags=xml_keep_tags,
        )

        partition_parameters_params.additional_properties = d
        return partition_parameters_params

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ExpiresAfter")


@_attrs_define
class ExpiresAfter:
    """The expiration policy for a vector store.

    Attributes:
        anchor (Literal['last_active_at']):
        days (int):
    """

    anchor: Literal["last_active_at"]
    days: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        anchor = self.anchor

        days = self.days

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "anchor": anchor,
                "days": days,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        anchor = cast(Literal["last_active_at"], d.pop("anchor"))
        if anchor != "last_active_at":
            raise ValueError(f"anchor must match const 'last_active_at', got '{anchor}'")

        days = d.pop("days")

        expires_after = cls(
            anchor=anchor,
            days=days,
        )

        expires_after.additional_properties = d
        return expires_after

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class FileSearchRankingOptionsRankerType0(str, Enum):
    AUTO = "auto"
    DEFAULT_2024_08_21 = "default_2024_08_21"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.mcp_doc_server_type import MCPDocServerType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.mcp_doc_request_headers_type_0 import MCPDocRequestHeadersType0


T = TypeVar("T", bound="MCPDoc")


@_attrs_define
class MCPDoc:
    """
    Attributes:
        server_name (str): The name of the model context server.
        server_type (MCPDocServerType):
        server_url (str): The URL of the model context server.
        user_id (str):
        field_id (str):
        updated_at (datetime.datetime | Unset):
        created_at (datetime.datetime | Unset):
        description (None | str | Unset): A brief description of the model context server.
        allowed_tools (list[str] | None | Unset): A list of tool names that are permitted to use this model context
            server.
        request_headers (MCPDocRequestHeadersType0 | None | Unset): Optional HTTP headers to include in requests (e.g.,
            Authorization, Content-Type).
        oauth_token (None | str | Unset): Optional OAuth token for authenticating with the model context server.
    """

    server_name: str
    server_type: MCPDocServerType
    server_url: str
    user_id: str
    field_id: str
    updated_at: datetime.datetime | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    description: None | str | Unset = UNSET
    allowed_tools: list[str] | None | Unset = UNSET
    request_headers: MCPDocRequestHeadersType0 | None | Unset = UNSET
    oauth_token: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.mcp_doc_request_headers_type_0 import MCPDocRequestHeadersType0

        server_name = self.server_name

        server_type = self.server_type.value

        server_url = self.server_url

        user_id = self.user_id

        field_id = self.field_id

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        allowed_tools: list[str] | None | Unset
        if isinstance(self.allowed_tools, Unset):
            allowed_tools = UNSET
        elif isinstance(self.allowed_tools, list):
            allowed_tools = self.allowed_tools

        else:
            allowed_tools = self.allowed_tools

        request_headers: dict[str, Any] | None | Unset
        if isinstance(self.request_headers, Unset):
            request_headers = UNSET
        elif isinstance(self.request_headers, MCPDocRequestHeadersType0):
            request_headers = self.request_headers.to_dict()
        else:
            request_headers = self.request_headers

        oauth_token: None | str | Unset
        if isinstance(self.oauth_token, Unset):
            oauth_token = UNSET
        else:
            oauth_token = self.oauth_token

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "server_name": server_name,
                "server_type": server_type,
                "server_url": server_url,
                "user_id": user_id,
                "_id": field_id,
            }
        )
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if description is not UNSET:
            field_dict["description"] = description
        if allowed_tools is not UNSET:
            field_dict["allowed_tools"] = allowed_tools
        if request_headers is not UNSET:
            field_dict["request_headers"] = request_headers
        if oauth_token is not UNSET:
            field_dict["oauth_token"] = oauth_token

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.mcp_doc_request_headers_type_0 import MCPDocRequestHeadersType0

        d = dict(src_dict)
        server_name = d.pop("server_name")

        server_type = MCPDocServerType(d.pop("server_type"))

        server_url = d.pop("server_url")

        user_id = d.pop("user_id")

        field_id = d.pop("_id")

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_allowed_tools(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                allowed_tools_type_0 = cast(list[str], data)

                return allowed_tools_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        allowed_tools = _parse_allowed_tools(d.pop("allowed_tools", UNSET))

        def _parse_request_headers(data: object) -> MCPDocRequestHeadersType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                request_headers_type_0 = MCPDocRequestHeadersType0.from_dict(data)

                return request_headers_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(MCPDocRequestHeadersType0 | None | Unset, data)

        request_headers = _parse_request_headers(d.pop("request_headers", UNSET))

        def _parse_oauth_token(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        oauth_token = _parse_oauth_token(d.pop("oauth_token", UNSET))

        mcp_doc = cls(
            server_name=server_name,
            server_type=server_type,
            server_url=server_url,
            user_id=user_id,
            field_id=field_id,
            updated_at=updated_at,
            created_at=created_at,
            description=description,
            allowed_tools=allowed_tools,
            request_headers=request_headers,
            oauth_token=oauth_token,
        )

        mcp_doc.additional_properties = d
        return mcp_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.body_async_run_action_v1_actions_id_async_run_post_input import (
        BodyAsyncRunActionV1ActionsIdAsyncRunPostInput,
    )
    from ..models.webhook import Webhook


T = TypeVar("T", bound="BodyAsyncRunActionV1ActionsIdAsyncRunPost")


@_attrs_define
class BodyAsyncRunActionV1ActionsIdAsyncRunPost:
    """
    Attributes:
        input_ (BodyAsyncRunActionV1ActionsIdAsyncRunPostInput):
        webhook (None | Unset | Webhook):
    """

    input_: BodyAsyncRunActionV1ActionsIdAsyncRunPostInput
    webhook: None | Unset | Webhook = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.webhook import Webhook

        input_ = self.input_.to_dict()

        webhook: dict[str, Any] | None | Unset
        if isinstance(self.webhook, Unset):
            webhook = UNSET
        elif isinstance(self.webhook, Webhook):
            webhook = self.webhook.to_dict()
        else:
            webhook = self.webhook

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "input": input_,
            }
        )
        if webhook is not UNSET:
            field_dict["webhook"] = webhook

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.body_async_run_action_v1_actions_id_async_run_post_input import (
            BodyAsyncRunActionV1ActionsIdAsyncRunPostInput,
        )
        from ..models.webhook import Webhook

        d = dict(src_dict)
        input_ = BodyAsyncRunActionV1ActionsIdAsyncRunPostInput.from_dict(d.pop("input"))

        def _parse_webhook(data: object) -> None | Unset | Webhook:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                webhook_type_0 = Webhook.from_dict(data)

                return webhook_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | Webhook, data)

        webhook = _parse_webhook(d.pop("webhook", UNSET))

        body_async_run_action_v1_actions_id_async_run_post = cls(
            input_=input_,
            webhook=webhook,
        )

        body_async_run_action_v1_actions_id_async_run_post.additional_properties = d
        return body_async_run_action_v1_actions_id_async_run_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_graph_doc_graph_type_0 import ActionGraphDocGraphType0
    from ..models.action_graph_doc_input_type_0 import ActionGraphDocInputType0
    from ..models.action_graph_doc_node_details import ActionGraphDocNodeDetails
    from ..models.action_graph_doc_nodes_type_0 import ActionGraphDocNodesType0
    from ..models.action_graph_doc_output_type_0 import ActionGraphDocOutputType0


T = TypeVar("T", bound="ActionGraphDoc")


@_attrs_define
class ActionGraphDoc:
    """
    Attributes:
        name (str):
        user_id (str):
        field_id (str):
        updated_at (datetime.datetime | Unset):
        version (float | Unset):  Default: 0.0.
        description (str | Unset):  Default: ''.
        user_manual (str | Unset):  Default: ''.
        nodes (ActionGraphDocNodesType0 | None | Unset): map of node_id to action_id
        node_details (ActionGraphDocNodeDetails | Unset): map of node_id to Node
        graph (ActionGraphDocGraphType0 | None | Unset): map of node_id to list of node_ids
        start_node_id (None | str | Unset):
        input_ (ActionGraphDocInputType0 | None | Unset):
        output (ActionGraphDocOutputType0 | None | Unset):
        is_saved (bool | Unset):  Default: False.
        is_published (bool | Unset):  Default: False.
        shared_with (list[str] | Unset):
        time_saved_per_file (float | None | Unset): Time saved per file in hours for dashboard workflow calculations
            Default: 0.1.
        dollar_cost_per_hour (float | None | Unset): Dollar cost per hour for dashboard workflow calculations Default:
            50.0.
        created_at (datetime.datetime | Unset):  Default: isoparse('2026-03-12T15:03:59.848584').
    """

    name: str
    user_id: str
    field_id: str
    updated_at: datetime.datetime | Unset = UNSET
    version: float | Unset = 0.0
    description: str | Unset = ""
    user_manual: str | Unset = ""
    nodes: ActionGraphDocNodesType0 | None | Unset = UNSET
    node_details: ActionGraphDocNodeDetails | Unset = UNSET
    graph: ActionGraphDocGraphType0 | None | Unset = UNSET
    start_node_id: None | str | Unset = UNSET
    input_: ActionGraphDocInputType0 | None | Unset = UNSET
    output: ActionGraphDocOutputType0 | None | Unset = UNSET
    is_saved: bool | Unset = False
    is_published: bool | Unset = False
    shared_with: list[str] | Unset = UNSET
    time_saved_per_file: float | None | Unset = 0.1
    dollar_cost_per_hour: float | None | Unset = 50.0
    created_at: datetime.datetime | Unset = isoparse("2026-03-12T15:03:59.848584")
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.action_graph_doc_graph_type_0 import ActionGraphDocGraphType0
        from ..models.action_graph_doc_input_type_0 import ActionGraphDocInputType0
        from ..models.action_graph_doc_nodes_type_0 import ActionGraphDocNodesType0
        from ..models.action_graph_doc_output_type_0 import ActionGraphDocOutputType0

        name = self.name

        user_id = self.user_id

        field_id = self.field_id

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        version = self.version

        description = self.description

        user_manual = self.user_manual

        nodes: dict[str, Any] | None | Unset
        if isinstance(self.nodes, Unset):
            nodes = UNSET
        elif isinstance(self.nodes, ActionGraphDocNodesType0):
            nodes = self.nodes.to_dict()
        else:
            nodes = self.nodes

        node_details: dict[str, Any] | Unset = UNSET
        if not isinstance(self.node_details, Unset):
            node_details = self.node_details.to_dict()

        graph: dict[str, Any] | None | Unset
        if isinstance(self.graph, Unset):
            graph = UNSET
        elif isinstance(self.graph, ActionGraphDocGraphType0):
            graph = self.graph.to_dict()
        else:
            graph = self.graph

        start_node_id: None | str | Unset
        if isinstance(self.start_node_id, Unset):
            start_node_id = UNSET
        else:
            start_node_id = self.start_node_id

        input_: dict[str, Any] | None | Unset
        if isinstance(self.input_, Unset):
            input_ = UNSET
        elif isinstance(self.input_, ActionGraphDocInputType0):
            input_ = self.input_.to_dict()
        else:
            input_ = self.input_

        output: dict[str, Any] | None | Unset
        if isinstance(self.output, Unset):
            output = UNSET
        elif isinstance(self.output, ActionGraphDocOutputType0):
            output = self.output.to_dict()
        else:
            output = self.output

        is_saved = self.is_saved

        is_published = self.is_published

        shared_with: list[str] | Unset = UNSET
        if not isinstance(self.shared_with, Unset):
            shared_with = self.shared_with

        time_saved_per_file: float | None | Unset
        if isinstance(self.time_saved_per_file, Unset):
            time_saved_per_file = UNSET
        else:
            time_saved_per_file = self.time_saved_per_file

        dollar_cost_per_hour: float | None | Unset
        if isinstance(self.dollar_cost_per_hour, Unset):
            dollar_cost_per_hour = UNSET
        else:
            dollar_cost_per_hour = self.dollar_cost_per_hour

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "user_id": user_id,
                "_id": field_id,
            }
        )
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if version is not UNSET:
            field_dict["version"] = version
        if description is not UNSET:
            field_dict["description"] = description
        if user_manual is not UNSET:
            field_dict["user_manual"] = user_manual
        if nodes is not UNSET:
            field_dict["nodes"] = nodes
        if node_details is not UNSET:
            field_dict["node_details"] = node_details
        if graph is not UNSET:
            field_dict["graph"] = graph
        if start_node_id is not UNSET:
            field_dict["start_node_id"] = start_node_id
        if input_ is not UNSET:
            field_dict["input"] = input_
        if output is not UNSET:
            field_dict["output"] = output
        if is_saved is not UNSET:
            field_dict["is_saved"] = is_saved
        if is_published is not UNSET:
            field_dict["is_published"] = is_published
        if shared_with is not UNSET:
            field_dict["shared_with"] = shared_with
        if time_saved_per_file is not UNSET:
            field_dict["time_saved_per_file"] = time_saved_per_file
        if dollar_cost_per_hour is not UNSET:
            field_dict["dollar_cost_per_hour"] = dollar_cost_per_hour
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.action_graph_doc_graph_type_0 import ActionGraphDocGraphType0
        from ..models.action_graph_doc_input_type_0 import ActionGraphDocInputType0
        from ..models.action_graph_doc_node_details import ActionGraphDocNodeDetails
        from ..models.action_graph_doc_nodes_type_0 import ActionGraphDocNodesType0
        from ..models.action_graph_doc_output_type_0 import ActionGraphDocOutputType0

        d = dict(src_dict)
        name = d.pop("name")

        user_id = d.pop("user_id")

        field_id = d.pop("_id")

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        version = d.pop("version", UNSET)

        description = d.pop("description", UNSET)

        user_manual = d.pop("user_manual", UNSET)

        def _parse_nodes(data: object) -> ActionGraphDocNodesType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                nodes_type_0 = ActionGraphDocNodesType0.from_dict(data)

                return nodes_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionGraphDocNodesType0 | None | Unset, data)

        nodes = _parse_nodes(d.pop("nodes", UNSET))

        _node_details = d.pop("node_details", UNSET)
        node_details: ActionGraphDocNodeDetails | Unset
        if isinstance(_node_details, Unset):
            node_details = UNSET
        else:
            node_details = ActionGraphDocNodeDetails.from_dict(_node_details)

        def _parse_graph(data: object) -> ActionGraphDocGraphType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                graph_type_0 = ActionGraphDocGraphType0.from_dict(data)

                return graph_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionGraphDocGraphType0 | None | Unset, data)

        graph = _parse_graph(d.pop("graph", UNSET))

        def _parse_start_node_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        start_node_id = _parse_start_node_id(d.pop("start_node_id", UNSET))

        def _parse_input_(data: object) -> ActionGraphDocInputType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_type_0 = ActionGraphDocInputType0.from_dict(data)

                return input_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionGraphDocInputType0 | None | Unset, data)

        input_ = _parse_input_(d.pop("input", UNSET))

        def _parse_output(data: object) -> ActionGraphDocOutputType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_type_0 = ActionGraphDocOutputType0.from_dict(data)

                return output_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionGraphDocOutputType0 | None | Unset, data)

        output = _parse_output(d.pop("output", UNSET))

        is_saved = d.pop("is_saved", UNSET)

        is_published = d.pop("is_published", UNSET)

        shared_with = cast(list[str], d.pop("shared_with", UNSET))

        def _parse_time_saved_per_file(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        time_saved_per_file = _parse_time_saved_per_file(d.pop("time_saved_per_file", UNSET))

        def _parse_dollar_cost_per_hour(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        dollar_cost_per_hour = _parse_dollar_cost_per_hour(d.pop("dollar_cost_per_hour", UNSET))

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        action_graph_doc = cls(
            name=name,
            user_id=user_id,
            field_id=field_id,
            updated_at=updated_at,
            version=version,
            description=description,
            user_manual=user_manual,
            nodes=nodes,
            node_details=node_details,
            graph=graph,
            start_node_id=start_node_id,
            input_=input_,
            output=output,
            is_saved=is_saved,
            is_published=is_published,
            shared_with=shared_with,
            time_saved_per_file=time_saved_per_file,
            dollar_cost_per_hour=dollar_cost_per_hour,
            created_at=created_at,
        )

        action_graph_doc.additional_properties = d
        return action_graph_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Region(str, Enum):
    ARABIA = "Arabia"
    ARABIA_EN = "Arabia_en"
    ARGENTINA = "Argentina"
    AUSTRALIA = "Australia"
    AUSTRIA = "Austria"
    BELGIUM_FR = "Belgium_fr"
    BELGIUM_NL = "Belgium_nl"
    BRAZIL = "Brazil"
    BULGARIA = "Bulgaria"
    CANADA_EN = "Canada_en"
    CANADA_FR = "Canada_fr"
    CATALAN = "Catalan"
    CHILE = "Chile"
    CHINA = "China"
    COLOMBIA = "Colombia"
    CROATIA = "Croatia"
    CZECH_REPUBLIC = "Czech_Republic"
    DENMARK = "Denmark"
    ESTONIA = "Estonia"
    FINLAND = "Finland"
    FRANCE = "France"
    GERMANY = "Germany"
    GREECE = "Greece"
    HONG_KONG = "Hong_Kong"
    HUNGARY = "Hungary"
    INDIA = "India"
    INDONESIA = "Indonesia"
    INDONESIA_EN = "Indonesia_en"
    IRELAND = "Ireland"
    ISRAEL = "Israel"
    ITALY = "Italy"
    JAPAN = "Japan"
    KOREA = "Korea"
    LATIN_AMERICA = "Latin_America"
    LATVIA = "Latvia"
    LITHUANIA = "Lithuania"
    MALAYSIA = "Malaysia"
    MALAYSIA_EN = "Malaysia_en"
    MEXICO = "Mexico"
    NETHERLANDS = "Netherlands"
    NEW_ZEALAND = "New_Zealand"
    NORWAY = "Norway"
    NO_REGION = "No_Region"
    PERU = "Peru"
    PHILIPPINES_EN = "Philippines_en"
    PHILIPPINES_TL = "Philippines_tl"
    POLAND = "Poland"
    PORTUGAL = "Portugal"
    ROMANIA = "Romania"
    RUSSIA = "Russia"
    SINGAPORE_EN = "Singapore_en"
    SLOVAK_REPUBLIC = "Slovak_Republic"
    SLOVENIA = "Slovenia"
    SOUTH_AFRICA = "South_Africa"
    SPAIN = "Spain"
    SWEDEN_SV = "Sweden_sv"
    SWITZERLAND_DE = "Switzerland_de"
    SWITZERLAND_FR = "Switzerland_fr"
    SWITZERLAND_IT = "Switzerland_it"
    TAIWAN = "Taiwan"
    THAILAND = "Thailand"
    TURKEY = "Turkey"
    UKRAINE = "Ukraine"
    UNITED_KINGDOM_EN = "United_Kingdom_en"
    UNITED_STATES_EN = "United_States_en"
    UNITED_STATES_ES = "United_States_es"
    VENEZUELA = "Venezuela"
    VIETNAM_VI = "Vietnam_vi"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.usage_input_tokens_details import UsageInputTokensDetails
    from ..models.usage_output_tokens_details import UsageOutputTokensDetails


T = TypeVar("T", bound="Usage")


@_attrs_define
class Usage:
    """For `gpt-image-1` only, the token usage information for the image generation.

    Attributes:
        input_tokens (int):
        input_tokens_details (UsageInputTokensDetails): The input tokens detailed information for the image generation.
        output_tokens (int):
        total_tokens (int):
        output_tokens_details (None | Unset | UsageOutputTokensDetails):
    """

    input_tokens: int
    input_tokens_details: UsageInputTokensDetails
    output_tokens: int
    total_tokens: int
    output_tokens_details: None | Unset | UsageOutputTokensDetails = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.usage_output_tokens_details import UsageOutputTokensDetails

        input_tokens = self.input_tokens

        input_tokens_details = self.input_tokens_details.to_dict()

        output_tokens = self.output_tokens

        total_tokens = self.total_tokens

        output_tokens_details: dict[str, Any] | None | Unset
        if isinstance(self.output_tokens_details, Unset):
            output_tokens_details = UNSET
        elif isinstance(self.output_tokens_details, UsageOutputTokensDetails):
            output_tokens_details = self.output_tokens_details.to_dict()
        else:
            output_tokens_details = self.output_tokens_details

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "input_tokens": input_tokens,
                "input_tokens_details": input_tokens_details,
                "output_tokens": output_tokens,
                "total_tokens": total_tokens,
            }
        )
        if output_tokens_details is not UNSET:
            field_dict["output_tokens_details"] = output_tokens_details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.usage_input_tokens_details import UsageInputTokensDetails
        from ..models.usage_output_tokens_details import UsageOutputTokensDetails

        d = dict(src_dict)
        input_tokens = d.pop("input_tokens")

        input_tokens_details = UsageInputTokensDetails.from_dict(d.pop("input_tokens_details"))

        output_tokens = d.pop("output_tokens")

        total_tokens = d.pop("total_tokens")

        def _parse_output_tokens_details(data: object) -> None | Unset | UsageOutputTokensDetails:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_tokens_details_type_0 = UsageOutputTokensDetails.from_dict(data)

                return output_tokens_details_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UsageOutputTokensDetails, data)

        output_tokens_details = _parse_output_tokens_details(d.pop("output_tokens_details", UNSET))

        usage = cls(
            input_tokens=input_tokens,
            input_tokens_details=input_tokens_details,
            output_tokens=output_tokens,
            total_tokens=total_tokens,
            output_tokens_details=output_tokens_details,
        )

        usage.additional_properties = d
        return usage

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.chat_doc_openai_provider import ChatDocOpenaiProvider
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_doc import ActionDoc
    from ..models.message import Message


T = TypeVar("T", bound="ChatDoc")


@_attrs_define
class ChatDoc:
    """
    Attributes:
        action (ActionDoc):
        messages (list[Message]):
        openai_provider (ChatDocOpenaiProvider):
        title (None | str):
        user_id (str):
        field_id (str):
        updated_at (datetime.datetime | Unset):
        created_at (datetime.datetime | Unset):  Default: isoparse('2026-03-12T15:04:00.062144').
    """

    action: ActionDoc
    messages: list[Message]
    openai_provider: ChatDocOpenaiProvider
    title: None | str
    user_id: str
    field_id: str
    updated_at: datetime.datetime | Unset = UNSET
    created_at: datetime.datetime | Unset = isoparse("2026-03-12T15:04:00.062144")
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action = self.action.to_dict()

        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)

        openai_provider = self.openai_provider.value

        title: None | str
        title = self.title

        user_id = self.user_id

        field_id = self.field_id

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "action": action,
                "messages": messages,
                "openai_provider": openai_provider,
                "title": title,
                "user_id": user_id,
                "_id": field_id,
            }
        )
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.action_doc import ActionDoc
        from ..models.message import Message

        d = dict(src_dict)
        action = ActionDoc.from_dict(d.pop("action"))

        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:
            messages_item = Message.from_dict(messages_item_data)

            messages.append(messages_item)

        openai_provider = ChatDocOpenaiProvider(d.pop("openai_provider"))

        def _parse_title(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        title = _parse_title(d.pop("title"))

        user_id = d.pop("user_id")

        field_id = d.pop("_id")

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        chat_doc = cls(
            action=action,
            messages=messages,
            openai_provider=openai_provider,
            title=title,
            user_id=user_id,
            field_id=field_id,
            updated_at=updated_at,
            created_at=created_at,
        )

        chat_doc.additional_properties = d
        return chat_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.dashboard_metric import DashboardMetric


T = TypeVar("T", bound="DashboardSummary")


@_attrs_define
class DashboardSummary:
    """Main dashboard summary

    Attributes:
        total_agents (DashboardMetric): Individual metric data point
        active_workflows (DashboardMetric): Individual metric data point
        scheduled_jobs (DashboardMetric): Individual metric data point
        total_workflow_runs (DashboardMetric): Individual metric data point
        time_saved (DashboardMetric): Individual metric data point
        dollars_saved (DashboardMetric): Individual metric data point
        roi_percentage (DashboardMetric): Individual metric data point
        additional_revenue (DashboardMetric): Individual metric data point
        last_updated (datetime.datetime):
    """

    total_agents: DashboardMetric
    active_workflows: DashboardMetric
    scheduled_jobs: DashboardMetric
    total_workflow_runs: DashboardMetric
    time_saved: DashboardMetric
    dollars_saved: DashboardMetric
    roi_percentage: DashboardMetric
    additional_revenue: DashboardMetric
    last_updated: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_agents = self.total_agents.to_dict()

        active_workflows = self.active_workflows.to_dict()

        scheduled_jobs = self.scheduled_jobs.to_dict()

        total_workflow_runs = self.total_workflow_runs.to_dict()

        time_saved = self.time_saved.to_dict()

        dollars_saved = self.dollars_saved.to_dict()

        roi_percentage = self.roi_percentage.to_dict()

        additional_revenue = self.additional_revenue.to_dict()

        last_updated = self.last_updated.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total_agents": total_agents,
                "active_workflows": active_workflows,
                "scheduled_jobs": scheduled_jobs,
                "total_workflow_runs": total_workflow_runs,
                "time_saved": time_saved,
                "dollars_saved": dollars_saved,
                "roi_percentage": roi_percentage,
                "additional_revenue": additional_revenue,
                "last_updated": last_updated,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.dashboard_metric import DashboardMetric

        d = dict(src_dict)
        total_agents = DashboardMetric.from_dict(d.pop("total_agents"))

        active_workflows = DashboardMetric.from_dict(d.pop("active_workflows"))

        scheduled_jobs = DashboardMetric.from_dict(d.pop("scheduled_jobs"))

        total_workflow_runs = DashboardMetric.from_dict(d.pop("total_workflow_runs"))

        time_saved = DashboardMetric.from_dict(d.pop("time_saved"))

        dollars_saved = DashboardMetric.from_dict(d.pop("dollars_saved"))

        roi_percentage = DashboardMetric.from_dict(d.pop("roi_percentage"))

        additional_revenue = DashboardMetric.from_dict(d.pop("additional_revenue"))

        last_updated = isoparse(d.pop("last_updated"))

        dashboard_summary = cls(
            total_agents=total_agents,
            active_workflows=active_workflows,
            scheduled_jobs=scheduled_jobs,
            total_workflow_runs=total_workflow_runs,
            time_saved=time_saved,
            dollars_saved=dollars_saved,
            roi_percentage=roi_percentage,
            additional_revenue=additional_revenue,
            last_updated=last_updated,
        )

        dashboard_summary.additional_properties = d
        return dashboard_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.images_response_background_type_0 import ImagesResponseBackgroundType0
from ..models.images_response_output_format_type_0 import ImagesResponseOutputFormatType0
from ..models.images_response_quality_type_0 import ImagesResponseQualityType0
from ..models.images_response_size_type_0 import ImagesResponseSizeType0
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.image import Image
    from ..models.usage import Usage


T = TypeVar("T", bound="ImagesResponse")


@_attrs_define
class ImagesResponse:
    """The response from the image generation endpoint.

    Attributes:
        created (int):
        background (ImagesResponseBackgroundType0 | None | Unset):
        data (list[Image] | None | Unset):
        output_format (ImagesResponseOutputFormatType0 | None | Unset):
        quality (ImagesResponseQualityType0 | None | Unset):
        size (ImagesResponseSizeType0 | None | Unset):
        usage (None | Unset | Usage):
    """

    created: int
    background: ImagesResponseBackgroundType0 | None | Unset = UNSET
    data: list[Image] | None | Unset = UNSET
    output_format: ImagesResponseOutputFormatType0 | None | Unset = UNSET
    quality: ImagesResponseQualityType0 | None | Unset = UNSET
    size: ImagesResponseSizeType0 | None | Unset = UNSET
    usage: None | Unset | Usage = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.usage import Usage

        created = self.created

        background: None | str | Unset
        if isinstance(self.background, Unset):
            background = UNSET
        elif isinstance(self.background, ImagesResponseBackgroundType0):
            background = self.background.value
        else:
            background = self.background

        data: list[dict[str, Any]] | None | Unset
        if isinstance(self.data, Unset):
            data = UNSET
        elif isinstance(self.data, list):
            data = []
            for data_type_0_item_data in self.data:
                data_type_0_item = data_type_0_item_data.to_dict()
                data.append(data_type_0_item)

        else:
            data = self.data

        output_format: None | str | Unset
        if isinstance(self.output_format, Unset):
            output_format = UNSET
        elif isinstance(self.output_format, ImagesResponseOutputFormatType0):
            output_format = self.output_format.value
        else:
            output_format = self.output_format

        quality: None | str | Unset
        if isinstance(self.quality, Unset):
            quality = UNSET
        elif isinstance(self.quality, ImagesResponseQualityType0):
            quality = self.quality.value
        else:
            quality = self.quality

        size: None | str | Unset
        if isinstance(self.size, Unset):
            size = UNSET
        elif isinstance(self.size, ImagesResponseSizeType0):
            size = self.size.value
        else:
            size = self.size

        usage: dict[str, Any] | None | Unset
        if isinstance(self.usage, Unset):
            usage = UNSET
        elif isinstance(self.usage, Usage):
            usage = self.usage.to_dict()
        else:
            usage = self.usage

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "created": created,
            }
        )
        if background is not UNSET:
            field_dict["background"] = background
        if data is not UNSET:
            field_dict["data"] = data
        if output_format is not UNSET:
            field_dict["output_format"] = output_format
        if quality is not UNSET:
            field_dict["quality"] = quality
        if size is not UNSET:
            field_dict["size"] = size
        if usage is not UNSET:
            field_dict["usage"] = usage

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.image import Image
        from ..models.usage import Usage

        d = dict(src_dict)
        created = d.pop("created")

        def _parse_background(data: object) -> ImagesResponseBackgroundType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                background_type_0 = ImagesResponseBackgroundType0(data)

                return background_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImagesResponseBackgroundType0 | None | Unset, data)

        background = _parse_background(d.pop("background", UNSET))

        def _parse_data(data: object) -> list[Image] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                data_type_0 = []
                _data_type_0 = data
                for data_type_0_item_data in _data_type_0:
                    data_type_0_item = Image.from_dict(data_type_0_item_data)

                    data_type_0.append(data_type_0_item)

                return data_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[Image] | None | Unset, data)

        data = _parse_data(d.pop("data", UNSET))

        def _parse_output_format(data: object) -> ImagesResponseOutputFormatType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                output_format_type_0 = ImagesResponseOutputFormatType0(data)

                return output_format_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImagesResponseOutputFormatType0 | None | Unset, data)

        output_format = _parse_output_format(d.pop("output_format", UNSET))

        def _parse_quality(data: object) -> ImagesResponseQualityType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                quality_type_0 = ImagesResponseQualityType0(data)

                return quality_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImagesResponseQualityType0 | None | Unset, data)

        quality = _parse_quality(d.pop("quality", UNSET))

        def _parse_size(data: object) -> ImagesResponseSizeType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                size_type_0 = ImagesResponseSizeType0(data)

                return size_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ImagesResponseSizeType0 | None | Unset, data)

        size = _parse_size(d.pop("size", UNSET))

        def _parse_usage(data: object) -> None | Unset | Usage:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                usage_type_0 = Usage.from_dict(data)

                return usage_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | Usage, data)

        usage = _parse_usage(d.pop("usage", UNSET))

        images_response = cls(
            created=created,
            background=background,
            data=data,
            output_format=output_format,
            quality=quality,
            size=size,
            usage=usage,
        )

        images_response.additional_properties = d
        return images_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.knowledge_base_role import KnowledgeBaseRole

T = TypeVar("T", bound="KnowledgeBaseMembersBulkRequest")


@_attrs_define
class KnowledgeBaseMembersBulkRequest:
    """
    Attributes:
        emails (list[str]): List of emails to add.
        role (KnowledgeBaseRole): Role within a knowledge base / workspace.
    """

    emails: list[str]
    role: KnowledgeBaseRole
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        emails = self.emails

        role = self.role.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "emails": emails,
                "role": role,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        emails = cast(list[str], d.pop("emails"))

        role = KnowledgeBaseRole(d.pop("role"))

        knowledge_base_members_bulk_request = cls(
            emails=emails,
            role=role,
        )

        knowledge_base_members_bulk_request.additional_properties = d
        return knowledge_base_members_bulk_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class LlmChatOpenaiConfigModelType1(str, Enum):
    COMPUTER_USE_PREVIEW = "computer-use-preview"
    DEEPSEEK_R1 = "deepseek-r1"
    GPT_4O = "gpt-4o"
    GPT_4O_0513 = "gpt-4o-0513"
    GPT_4O_0806 = "gpt-4o-0806"
    GPT_4O_1120 = "gpt-4o-1120"
    GPT_4O_MINI = "gpt-4o-mini"
    GPT_4_1 = "gpt-4.1"
    GPT_4_1_MINI = "gpt-4.1-mini"
    GPT_4_1_NANO = "gpt-4.1-nano"
    GPT_4_32K = "gpt-4-32k"
    MODEL_ROUTER = "model-router"
    O1 = "o1"
    O1_MINI = "o1-mini"
    O3 = "o3"
    O3_MINI = "o3-mini"
    O3_PRO = "o3-pro"
    O4_MINI = "o4-mini"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.audio import Audio
    from ..models.chat_completion_content_part_refusal_param import ChatCompletionContentPartRefusalParam
    from ..models.chat_completion_content_part_text_param import ChatCompletionContentPartTextParam
    from ..models.chat_completion_message_custom_tool_call_param import ChatCompletionMessageCustomToolCallParam
    from ..models.chat_completion_message_function_tool_call_param import ChatCompletionMessageFunctionToolCallParam
    from ..models.function_call import FunctionCall


T = TypeVar("T", bound="ChatCompletionAssistantMessageParam")


@_attrs_define
class ChatCompletionAssistantMessageParam:
    """Messages sent by the model in response to user messages.

    Attributes:
        role (Literal['assistant']):
        audio (Audio | None | Unset):
        content (list[ChatCompletionContentPartRefusalParam | ChatCompletionContentPartTextParam] | None | str | Unset):
        function_call (FunctionCall | None | Unset):
        name (str | Unset):
        refusal (None | str | Unset):
        tool_calls (list[ChatCompletionMessageCustomToolCallParam | ChatCompletionMessageFunctionToolCallParam] |
            Unset):
    """

    role: Literal["assistant"]
    audio: Audio | None | Unset = UNSET
    content: list[ChatCompletionContentPartRefusalParam | ChatCompletionContentPartTextParam] | None | str | Unset = (
        UNSET
    )
    function_call: FunctionCall | None | Unset = UNSET
    name: str | Unset = UNSET
    refusal: None | str | Unset = UNSET
    tool_calls: list[ChatCompletionMessageCustomToolCallParam | ChatCompletionMessageFunctionToolCallParam] | Unset = (
        UNSET
    )
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.audio import Audio
        from ..models.chat_completion_content_part_text_param import ChatCompletionContentPartTextParam
        from ..models.chat_completion_message_function_tool_call_param import ChatCompletionMessageFunctionToolCallParam
        from ..models.function_call import FunctionCall

        role = self.role

        audio: dict[str, Any] | None | Unset
        if isinstance(self.audio, Unset):
            audio = UNSET
        elif isinstance(self.audio, Audio):
            audio = self.audio.to_dict()
        else:
            audio = self.audio

        content: list[dict[str, Any]] | None | str | Unset
        if isinstance(self.content, Unset):
            content = UNSET
        elif isinstance(self.content, list):
            content = []
            for content_type_1_item_data in self.content:
                content_type_1_item: dict[str, Any]
                if isinstance(content_type_1_item_data, ChatCompletionContentPartTextParam):
                    content_type_1_item = content_type_1_item_data.to_dict()
                else:
                    content_type_1_item = content_type_1_item_data.to_dict()

                content.append(content_type_1_item)

        else:
            content = self.content

        function_call: dict[str, Any] | None | Unset
        if isinstance(self.function_call, Unset):
            function_call = UNSET
        elif isinstance(self.function_call, FunctionCall):
            function_call = self.function_call.to_dict()
        else:
            function_call = self.function_call

        name = self.name

        refusal: None | str | Unset
        if isinstance(self.refusal, Unset):
            refusal = UNSET
        else:
            refusal = self.refusal

        tool_calls: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.tool_calls, Unset):
            tool_calls = []
            for tool_calls_item_data in self.tool_calls:
                tool_calls_item: dict[str, Any]
                if isinstance(tool_calls_item_data, ChatCompletionMessageFunctionToolCallParam):
                    tool_calls_item = tool_calls_item_data.to_dict()
                else:
                    tool_calls_item = tool_calls_item_data.to_dict()

                tool_calls.append(tool_calls_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "role": role,
            }
        )
        if audio is not UNSET:
            field_dict["audio"] = audio
        if content is not UNSET:
            field_dict["content"] = content
        if function_call is not UNSET:
            field_dict["function_call"] = function_call
        if name is not UNSET:
            field_dict["name"] = name
        if refusal is not UNSET:
            field_dict["refusal"] = refusal
        if tool_calls is not UNSET:
            field_dict["tool_calls"] = tool_calls

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.audio import Audio
        from ..models.chat_completion_content_part_refusal_param import ChatCompletionContentPartRefusalParam
        from ..models.chat_completion_content_part_text_param import ChatCompletionContentPartTextParam
        from ..models.chat_completion_message_custom_tool_call_param import ChatCompletionMessageCustomToolCallParam
        from ..models.chat_completion_message_function_tool_call_param import ChatCompletionMessageFunctionToolCallParam
        from ..models.function_call import FunctionCall

        d = dict(src_dict)
        role = cast(Literal["assistant"], d.pop("role"))
        if role != "assistant":
            raise ValueError(f"role must match const 'assistant', got '{role}'")

        def _parse_audio(data: object) -> Audio | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                audio_type_0 = Audio.from_dict(data)

                return audio_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(Audio | None | Unset, data)

        audio = _parse_audio(d.pop("audio", UNSET))

        def _parse_content(
            data: object,
        ) -> list[ChatCompletionContentPartRefusalParam | ChatCompletionContentPartTextParam] | None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                content_type_1 = []
                _content_type_1 = data
                for content_type_1_item_data in _content_type_1:

                    def _parse_content_type_1_item(
                        data: object,
                    ) -> ChatCompletionContentPartRefusalParam | ChatCompletionContentPartTextParam:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            content_type_1_item_type_0 = ChatCompletionContentPartTextParam.from_dict(data)

                            return content_type_1_item_type_0
                        except (TypeError, ValueError, AttributeError, KeyError):
                            pass
                        if not isinstance(data, dict):
                            raise TypeError()
                        content_type_1_item_type_1 = ChatCompletionContentPartRefusalParam.from_dict(data)

                        return content_type_1_item_type_1

                    content_type_1_item = _parse_content_type_1_item(content_type_1_item_data)

                    content_type_1.append(content_type_1_item)

                return content_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(
                list[ChatCompletionContentPartRefusalParam | ChatCompletionContentPartTextParam] | None | str | Unset,
                data,
            )

        content = _parse_content(d.pop("content", UNSET))

        def _parse_function_call(data: object) -> FunctionCall | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                function_call_type_0 = FunctionCall.from_dict(data)

                return function_call_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(FunctionCall | None | Unset, data)

        function_call = _parse_function_call(d.pop("function_call", UNSET))

        name = d.pop("name", UNSET)

        def _parse_refusal(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        refusal = _parse_refusal(d.pop("refusal", UNSET))

        _tool_calls = d.pop("tool_calls", UNSET)
        tool_calls: (
            list[ChatCompletionMessageCustomToolCallParam | ChatCompletionMessageFunctionToolCallParam] | Unset
        ) = UNSET
        if _tool_calls is not UNSET:
            tool_calls = []
            for tool_calls_item_data in _tool_calls:

                def _parse_tool_calls_item(
                    data: object,
                ) -> ChatCompletionMessageCustomToolCallParam | ChatCompletionMessageFunctionToolCallParam:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        tool_calls_item_type_0 = ChatCompletionMessageFunctionToolCallParam.from_dict(data)

                        return tool_calls_item_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    tool_calls_item_type_1 = ChatCompletionMessageCustomToolCallParam.from_dict(data)

                    return tool_calls_item_type_1

                tool_calls_item = _parse_tool_calls_item(tool_calls_item_data)

                tool_calls.append(tool_calls_item)

        chat_completion_assistant_message_param = cls(
            role=role,
            audio=audio,
            content=content,
            function_call=function_call,
            name=name,
            refusal=refusal,
            tool_calls=tool_calls,
        )

        chat_completion_assistant_message_param.additional_properties = d
        return chat_completion_assistant_message_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BulkMemberResponse")


@_attrs_define
class BulkMemberResponse:
    """
    Attributes:
        total_requested (int):
        inserted_count (int):
        updated_count (int):
        matched_count (int):
        invalid_emails (list[str] | Unset):
        not_found_emails (list[str] | Unset):
        skipped_owner_emails (list[str] | Unset):
        skipped_legacy_users (list[str] | Unset):
    """

    total_requested: int
    inserted_count: int
    updated_count: int
    matched_count: int
    invalid_emails: list[str] | Unset = UNSET
    not_found_emails: list[str] | Unset = UNSET
    skipped_owner_emails: list[str] | Unset = UNSET
    skipped_legacy_users: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_requested = self.total_requested

        inserted_count = self.inserted_count

        updated_count = self.updated_count

        matched_count = self.matched_count

        invalid_emails: list[str] | Unset = UNSET
        if not isinstance(self.invalid_emails, Unset):
            invalid_emails = self.invalid_emails

        not_found_emails: list[str] | Unset = UNSET
        if not isinstance(self.not_found_emails, Unset):
            not_found_emails = self.not_found_emails

        skipped_owner_emails: list[str] | Unset = UNSET
        if not isinstance(self.skipped_owner_emails, Unset):
            skipped_owner_emails = self.skipped_owner_emails

        skipped_legacy_users: list[str] | Unset = UNSET
        if not isinstance(self.skipped_legacy_users, Unset):
            skipped_legacy_users = self.skipped_legacy_users

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total_requested": total_requested,
                "inserted_count": inserted_count,
                "updated_count": updated_count,
                "matched_count": matched_count,
            }
        )
        if invalid_emails is not UNSET:
            field_dict["invalid_emails"] = invalid_emails
        if not_found_emails is not UNSET:
            field_dict["not_found_emails"] = not_found_emails
        if skipped_owner_emails is not UNSET:
            field_dict["skipped_owner_emails"] = skipped_owner_emails
        if skipped_legacy_users is not UNSET:
            field_dict["skipped_legacy_users"] = skipped_legacy_users

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        total_requested = d.pop("total_requested")

        inserted_count = d.pop("inserted_count")

        updated_count = d.pop("updated_count")

        matched_count = d.pop("matched_count")

        invalid_emails = cast(list[str], d.pop("invalid_emails", UNSET))

        not_found_emails = cast(list[str], d.pop("not_found_emails", UNSET))

        skipped_owner_emails = cast(list[str], d.pop("skipped_owner_emails", UNSET))

        skipped_legacy_users = cast(list[str], d.pop("skipped_legacy_users", UNSET))

        bulk_member_response = cls(
            total_requested=total_requested,
            inserted_count=inserted_count,
            updated_count=updated_count,
            matched_count=matched_count,
            invalid_emails=invalid_emails,
            not_found_emails=not_found_emails,
            skipped_owner_emails=skipped_owner_emails,
            skipped_legacy_users=skipped_legacy_users,
        )

        bulk_member_response.additional_properties = d
        return bulk_member_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.metric_type import MetricType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.time_series_data_point import TimeSeriesDataPoint


T = TypeVar("T", bound="ChartData")


@_attrs_define
class ChartData:
    """Chart data structure

    Attributes:
        title (str):
        data_points (list[TimeSeriesDataPoint]):
        metric_type (MetricType):
        unit (str | Unset):  Default: ''.
    """

    title: str
    data_points: list[TimeSeriesDataPoint]
    metric_type: MetricType
    unit: str | Unset = ""
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        data_points = []
        for data_points_item_data in self.data_points:
            data_points_item = data_points_item_data.to_dict()
            data_points.append(data_points_item)

        metric_type = self.metric_type.value

        unit = self.unit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "title": title,
                "data_points": data_points,
                "metric_type": metric_type,
            }
        )
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.time_series_data_point import TimeSeriesDataPoint

        d = dict(src_dict)
        title = d.pop("title")

        data_points = []
        _data_points = d.pop("data_points")
        for data_points_item_data in _data_points:
            data_points_item = TimeSeriesDataPoint.from_dict(data_points_item_data)

            data_points.append(data_points_item)

        metric_type = MetricType(d.pop("metric_type"))

        unit = d.pop("unit", UNSET)

        chart_data = cls(
            title=title,
            data_points=data_points,
            metric_type=metric_type,
            unit=unit,
        )

        chart_data.additional_properties = d
        return chart_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListIngestedFilesProxyV1IngestionFilesGetSortBy(str, Enum):
    CREATED_AT = "created_at"
    ORIGINAL_FILENAME = "original_filename"
    STATUS = "status"
    UPDATED_AT = "updated_at"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeSeriesDataPoint")


@_attrs_define
class TimeSeriesDataPoint:
    """Data point for time series charts

    Attributes:
        date (datetime.datetime):
        value (float):
        label (None | str | Unset):
    """

    date: datetime.datetime
    value: float
    label: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        date = self.date.isoformat()

        value = self.value

        label: None | str | Unset
        if isinstance(self.label, Unset):
            label = UNSET
        else:
            label = self.label

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "date": date,
                "value": value,
            }
        )
        if label is not UNSET:
            field_dict["label"] = label

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        date = isoparse(d.pop("date"))

        value = d.pop("value")

        def _parse_label(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        label = _parse_label(d.pop("label", UNSET))

        time_series_data_point = cls(
            date=date,
            value=value,
            label=label,
        )

        time_series_data_point.additional_properties = d
        return time_series_data_point

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="IntervalTriggerModel")


@_attrs_define
class IntervalTriggerModel:
    """
    Attributes:
        trigger (Literal['interval'] | Unset):  Default: 'interval'.
        seconds (int | Unset):  Default: 86400.
    """

    trigger: Literal["interval"] | Unset = "interval"
    seconds: int | Unset = 86400
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        trigger = self.trigger

        seconds = self.seconds

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if trigger is not UNSET:
            field_dict["trigger"] = trigger
        if seconds is not UNSET:
            field_dict["seconds"] = seconds

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        trigger = cast(Literal["interval"] | Unset, d.pop("trigger", UNSET))
        if trigger != "interval" and not isinstance(trigger, Unset):
            raise ValueError(f"trigger must match const 'interval', got '{trigger}'")

        seconds = d.pop("seconds", UNSET)

        interval_trigger_model = cls(
            trigger=trigger,
            seconds=seconds,
        )

        interval_trigger_model.additional_properties = d
        return interval_trigger_model

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.time_range_config import TimeRangeConfig


T = TypeVar("T", bound="CostConfigUpdate")


@_attrs_define
class CostConfigUpdate:
    """Model for updating cost configuration

    Attributes:
        value (float | None | Unset): The configuration value
        description (None | str | Unset): Description of this configuration
        time_ranges (None | TimeRangeConfig | Unset): Time range configuration for time saved calculations
    """

    value: float | None | Unset = UNSET
    description: None | str | Unset = UNSET
    time_ranges: None | TimeRangeConfig | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.time_range_config import TimeRangeConfig

        value: float | None | Unset
        if isinstance(self.value, Unset):
            value = UNSET
        else:
            value = self.value

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        time_ranges: dict[str, Any] | None | Unset
        if isinstance(self.time_ranges, Unset):
            time_ranges = UNSET
        elif isinstance(self.time_ranges, TimeRangeConfig):
            time_ranges = self.time_ranges.to_dict()
        else:
            time_ranges = self.time_ranges

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if value is not UNSET:
            field_dict["value"] = value
        if description is not UNSET:
            field_dict["description"] = description
        if time_ranges is not UNSET:
            field_dict["time_ranges"] = time_ranges

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.time_range_config import TimeRangeConfig

        d = dict(src_dict)

        def _parse_value(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        value = _parse_value(d.pop("value", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_time_ranges(data: object) -> None | TimeRangeConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                time_ranges_type_0 = TimeRangeConfig.from_dict(data)

                return time_ranges_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TimeRangeConfig | Unset, data)

        time_ranges = _parse_time_ranges(d.pop("time_ranges", UNSET))

        cost_config_update = cls(
            value=value,
            description=description,
            time_ranges=time_ranges,
        )

        cost_config_update.additional_properties = d
        return cost_config_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.chat_doc import ChatDoc
from ...models.create_chat_v1_action_chats_post_openai_provider import CreateChatV1ActionChatsPostOpenaiProvider
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response


def _get_kwargs(
    *,
    action_id: str,
    openai_provider: CreateChatV1ActionChatsPostOpenaiProvider,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["action_id"] = action_id

    json_openai_provider = openai_provider.value
    params["openai_provider"] = json_openai_provider

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/action_chats/",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ChatDoc | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = ChatDoc.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ChatDoc | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    action_id: str,
    openai_provider: CreateChatV1ActionChatsPostOpenaiProvider,
) -> Response[ChatDoc | HTTPValidationError]:
    """Create Chat

    Args:
        action_id (str):
        openai_provider (CreateChatV1ActionChatsPostOpenaiProvider):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ChatDoc | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        action_id=action_id,
        openai_provider=openai_provider,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    action_id: str,
    openai_provider: CreateChatV1ActionChatsPostOpenaiProvider,
) -> ChatDoc | HTTPValidationError | None:
    """Create Chat

    Args:
        action_id (str):
        openai_provider (CreateChatV1ActionChatsPostOpenaiProvider):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ChatDoc | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        action_id=action_id,
        openai_provider=openai_provider,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    action_id: str,
    openai_provider: CreateChatV1ActionChatsPostOpenaiProvider,
) -> Response[ChatDoc | HTTPValidationError]:
    """Create Chat

    Args:
        action_id (str):
        openai_provider (CreateChatV1ActionChatsPostOpenaiProvider):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ChatDoc | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        action_id=action_id,
        openai_provider=openai_provider,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    action_id: str,
    openai_provider: CreateChatV1ActionChatsPostOpenaiProvider,
) -> ChatDoc | HTTPValidationError | None:
    """Create Chat

    Args:
        action_id (str):
        openai_provider (CreateChatV1ActionChatsPostOpenaiProvider):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ChatDoc | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            action_id=action_id,
            openai_provider=openai_provider,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.cron_schedule import CronSchedule
    from ..models.data_block import DataBlock
    from ..models.interval_trigger_model import IntervalTriggerModel


T = TypeVar("T", bound="ScheduleUpdate")


@_attrs_define
class ScheduleUpdate:
    """
    Attributes:
        name (None | str | Unset):
        trigger (CronSchedule | IntervalTriggerModel | None | Unset):
        runnable_id (None | str | Unset):
        input_ (DataBlock | None | Unset):
        run_id (None | str | Unset):
    """

    name: None | str | Unset = UNSET
    trigger: CronSchedule | IntervalTriggerModel | None | Unset = UNSET
    runnable_id: None | str | Unset = UNSET
    input_: DataBlock | None | Unset = UNSET
    run_id: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.cron_schedule import CronSchedule
        from ..models.data_block import DataBlock
        from ..models.interval_trigger_model import IntervalTriggerModel

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        trigger: dict[str, Any] | None | Unset
        if isinstance(self.trigger, Unset):
            trigger = UNSET
        elif isinstance(self.trigger, CronSchedule):
            trigger = self.trigger.to_dict()
        elif isinstance(self.trigger, IntervalTriggerModel):
            trigger = self.trigger.to_dict()
        else:
            trigger = self.trigger

        runnable_id: None | str | Unset
        if isinstance(self.runnable_id, Unset):
            runnable_id = UNSET
        else:
            runnable_id = self.runnable_id

        input_: dict[str, Any] | None | Unset
        if isinstance(self.input_, Unset):
            input_ = UNSET
        elif isinstance(self.input_, DataBlock):
            input_ = self.input_.to_dict()
        else:
            input_ = self.input_

        run_id: None | str | Unset
        if isinstance(self.run_id, Unset):
            run_id = UNSET
        else:
            run_id = self.run_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if trigger is not UNSET:
            field_dict["trigger"] = trigger
        if runnable_id is not UNSET:
            field_dict["runnable_id"] = runnable_id
        if input_ is not UNSET:
            field_dict["input"] = input_
        if run_id is not UNSET:
            field_dict["run_id"] = run_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.cron_schedule import CronSchedule
        from ..models.data_block import DataBlock
        from ..models.interval_trigger_model import IntervalTriggerModel

        d = dict(src_dict)

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_trigger(data: object) -> CronSchedule | IntervalTriggerModel | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                trigger_type_0 = CronSchedule.from_dict(data)

                return trigger_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                trigger_type_1 = IntervalTriggerModel.from_dict(data)

                return trigger_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(CronSchedule | IntervalTriggerModel | None | Unset, data)

        trigger = _parse_trigger(d.pop("trigger", UNSET))

        def _parse_runnable_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        runnable_id = _parse_runnable_id(d.pop("runnable_id", UNSET))

        def _parse_input_(data: object) -> DataBlock | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_type_0 = DataBlock.from_dict(data)

                return input_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DataBlock | None | Unset, data)

        input_ = _parse_input_(d.pop("input", UNSET))

        def _parse_run_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        run_id = _parse_run_id(d.pop("run_id", UNSET))

        schedule_update = cls(
            name=name,
            trigger=trigger,
            runnable_id=runnable_id,
            input_=input_,
            run_id=run_id,
        )

        schedule_update.additional_properties = d
        return schedule_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
